package com.qs.mp.channel.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.LambdaUpdateWrapper;
import com.qs.mp.channel.domain.ChannelGoods;
import com.qs.mp.channel.domain.ChannelGoodsSettleOrder;
import com.qs.mp.channel.domain.ChannelGoodsSettleOrderItem;
import com.qs.mp.channel.domain.param.PushChannelGoodsSettleParam;
import com.qs.mp.channel.mapper.ChannelGoodsSettleOrderMapper;
import com.qs.mp.channel.service.IChannelGoodsService;
import com.qs.mp.channel.service.IChannelGoodsSettleOrderItemService;
import com.qs.mp.channel.service.IChannelGoodsSettleOrderService;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.qs.mp.common.enums.ChannelGoodsOrderSettleEnum;
import com.qs.mp.common.enums.ChannelGoodsSettleStatusTypeEnum;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.Assert;

import java.util.List;

/**
 * <p>
 * 经销商商品结算订单 服务实现类
 * </p>
 *
 * @author quanshu
 * @since 2023-06-08
 */
@Service
public class ChannelGoodsSettleOrderServiceImpl extends ServiceImpl<ChannelGoodsSettleOrderMapper, ChannelGoodsSettleOrder> implements IChannelGoodsSettleOrderService {

    @Autowired
    private IChannelGoodsService channelGoodsService;

    @Autowired
    private IChannelGoodsSettleOrderService settleOrderService;

    @Autowired
    private IChannelGoodsSettleOrderItemService settleOrderItemService;

    @Override
    @Transactional(rollbackFor = Exception.class)
    public Long pushSettleOrder(PushChannelGoodsSettleParam settleVO) {
        List<ChannelGoods> channelGoodsList = channelGoodsService.list(new LambdaQueryWrapper<ChannelGoods>().eq(ChannelGoods::getOrderId, settleVO.getOrderId()));
        ChannelGoodsSettleOrder settleOrder = new ChannelGoodsSettleOrder();
        settleOrder.setSettleNum(channelGoodsList.stream().mapToInt(ChannelGoods::getSettleQty).sum());
        Integer orderAmt = 0;
        for (ChannelGoods channelGoods : channelGoodsList) {
            orderAmt += channelGoods.getPurchaseCost() * channelGoods.getSettleQty();
        }
        settleOrder.setChannelId(channelGoodsList.get(0).getChannelId());
        settleOrder.setOrderAmt(orderAmt);
        settleOrder.setSettleAmt(settleVO.getSettleAmt());
        settleOrder.setPayType("微信");
        settleOrder.setStatus(ChannelGoodsOrderSettleEnum.NOT_PAY);
        settleOrderService.save(settleOrder);

        for (ChannelGoods channelGoods : channelGoodsList) {
            ChannelGoodsSettleOrderItem settleOrderItem = new ChannelGoodsSettleOrderItem();
            settleOrderItem.setOrderId(settleOrder.getOrderId());
            settleOrderItem.setChannelId(channelGoods.getChannelId());
            settleOrderItem.setChannelGoodsId(channelGoods.getId());
            settleOrderItem.setTitle(channelGoods.getTitle());
            settleOrderItem.setPicUrl(channelGoods.getPicUrl());
            settleOrderItem.setQuantity(channelGoods.getQuantity());
            settleOrderItem.setPurchaseCost(channelGoods.getPurchaseCost());
            settleOrderItem.setVerifyQty(channelGoods.getVerifyQty());
            settleOrderItem.setRemainQty(channelGoods.getRemainQty());
            settleOrderItem.setSettleQty(channelGoods.getSettleQty());
            settleOrderItemService.save(settleOrderItem);
        }

        boolean update = channelGoodsService.update(new LambdaUpdateWrapper<ChannelGoods>().set(ChannelGoods::getStatus, ChannelGoodsSettleStatusTypeEnum.DOING).eq(ChannelGoods::getOrderId, settleVO.getOrderId()));
        Assert.isTrue(update, "更新门店采购商品信息失败：order_id =" + settleVO.getOrderId());

        return settleOrder.getOrderId();
    }
}
